<?php

 /* This plugin will handle the most common attachments that are
  * found -- text attachments and images that the browser can handle.
  */

global $FileExtensionToMimeType;
$FileExtensionToMimeType = array('bmp' => 'image/x-bitmap',
                  'gif' => 'image/gif',
		  'htm' => 'text/html',
		  'html' => 'text/html',
		  'jpg' => 'image/jpeg',
		  'jpeg' => 'image/jpeg',
		  'php' => 'text/plain',
		  'png' => 'image/png',
		  'rtf' => 'text/richtext',
		  'txt' => 'text/plain',
		  'vcf' => 'text/x-vcard');

function squirrelmail_plugin_init_attachment_common($debug = 0)
{
  global $squirrelmail_plugin_hooks, $HTTP_SERVER_VARS, $HTTP_ACCEPT,
     $attachment_common_types, $attachment_common_types_parsed;

  $debug=0;

  // Register this plugin for supported images
  if (! isset($attachment_common_types)) {
     $attachment_common_types = array();
     $attachment_common_types_parsed = array();
     session_register('attachment_common_types');
     session_register('attachment_common_types_parsed');
  }
  
  if (isset($HTTP_SERVER_VARS['HTTP_ACCEPT']) &&
      !isset($attachment_common_types_parsed[$HTTP_SERVER_VARS['HTTP_ACCEPT']]))
      attachment_common_parse($HTTP_SERVER_VARS['HTTP_ACCEPT'], $debug);
  if (isset($HTTP_ACCEPT) &&
      !isset($attachment_common_types_parsed[$HTTP_ACCEPT]))
      attachment_common_parse($HTTP_ACCEPT, $debug);
    
  if ($debug)
  {
     echo '<pre>';
     ob_start();
     echo "attachment_common :: attachment_common_types =\n";
     var_dump($attachment_common_types);
     echo "attachment_common :: attachment_common_types_parsed =\n";
     var_dump($attachment_common_types_parsed);
     $contents = ob_get_contents();
     ob_end_clean();
     echo htmlspecialchars($contents);
     echo "\n</pre>";
  }

  foreach ($attachment_common_types as $val => $v) {
     if ($debug)
        echo "attachment_common :: registering $val<br>\n";
     if ($val == 'image/gif')
        $squirrelmail_plugin_hooks['attachment image/gif']['attachment_common'] = 'attachment_common_link_image';
     elseif ($val == 'image/jpeg' || $val == 'image/pjpeg') {
        $squirrelmail_plugin_hooks['attachment image/jpeg']['attachment_common'] = 'attachment_common_link_image';
        $squirrelmail_plugin_hooks['attachment image/pjpeg']['attachment_common'] = 'attachment_common_link_image';
     }
     elseif ($val == 'image/png')
        $squirrelmail_plugin_hooks['attachment image/png']['attachment_common'] = 'attachment_common_link_image';
     elseif ($val == 'image/x-xbitmap')
        $squirrelmail_plugin_hooks['attachment image/x-xbitmap']['attachment_common'] = 'attachment_common_link_image';
     elseif ($debug)
        echo "attachment_common :: couldn't register $val<br>\n";
  }

  if ($debug)
  {
     echo '<pre>';
     ob_start();
     echo "attachment_common :: squirrelmail_plugin_hooks =\n";
     var_dump($squirrelmail_plugin_hooks);
     $contents = ob_get_contents();
     ob_end_clean();
     echo htmlspecialchars($contents);
     echo "\n</pre>";
  }

  // Register this plugin for text-type attachments
  $squirrelmail_plugin_hooks['attachment message/rfc822']['attachment_common'] = 'attachment_common_link_text';
  $squirrelmail_plugin_hooks['attachment text/plain']['attachment_common'] = 'attachment_common_link_text';
  $squirrelmail_plugin_hooks['attachment text/richtext']['attachment_common'] = 'attachment_common_link_text';

  // HTML
  $squirrelmail_plugin_hooks['attachment text/html']['attachment_common'] = 'attachment_common_link_html';

  // vcards
  $squirrelmail_plugin_hooks['attachment text/x-vcard']['attachment_common'] = 'attachment_common_link_vcard';
  
  // "unknown" attachments
  $squirrelmail_plugin_hooks['attachment application/octet-stream']['attachment_common'] = 'attachment_common_octet_stream';
  
  // Display images at bottom (optional)
  $squirrelmail_plugin_hooks['read_body_bottom']['attachment_common'] =
     'attachment_common_read_body_bottom';
  
  // Options (display and save -- will load manually)
  $squirrelmail_plugin_hooks['options_display_inside']['attachment_common'] =
     'attachment_common_options';
  $squirrelmail_plugin_hooks['options_display_save']['attachment_common'] =
     'attachment_common_options_save';
}


function attachment_common_parse($str, $debug)
{
   global $attachment_common_types, $attachment_common_types_parsed;
   
   $attachment_common_types_parsed[$str] = true;
   $types = explode(', ', $str);

   foreach ($types as $val)
   {
      // Ignore the ";q=1.0" stuff
      if (strpos($val, ';') !== false)
         $val = substr($val, 0, strpos($val, ';'));
      
      if (! isset($attachment_common_types[$val])) {
         if ($debug)
             echo "plugin attachment_common:  Registering $val<br>\n";
	 $attachment_common_types[$val] = true;
      }
   }
}


function attachment_common_link_text(&$Args)
{
  // If there is a text attachment, we would like to create a 'view' button
  // that links to the text attachment viewer.
  //
  // $Args[1] = the array of actions
  //
  // Use our plugin name for adding an action
  // $Args[1]['attachment_common'] = array for href and text
  //
  // $Args[1]['attachment_common']['text'] = What is displayed
  // $Args[1]['attachment_common']['href'] = Where it links to
  //
  // This sets the 'href' of this plugin for a new link.
  $Args[1]['attachment_common']['href'] = '../src/download.php?startMessage=' . 
     $Args[2] . '&passed_id=' . $Args[3] . '&mailbox=' . $Args[4] . '&passed_ent_id=' .
     $Args[5] . '&override_type0=text&override_type1=plain';
  
  // If we got here from a search, we should preserve these variables
  if ($Args[8] && $Args[9])
     $Args[1]['attachment_common']['href'] .= '&where=' . 
     urlencode($Args[8]) . '&what=' . urlencode($Args[9]);

  // The link that we created needs a name.  "view" will be displayed for
  // all text attachments handled by this plugin.
  $Args[1]['attachment_common']['text'] = _("view");
  
  // Each attachment has a filename on the left, which is a link.
  // Where that link points to can be changed.  Just in case the link above
  // for viewing text attachments is not the same as the default link for
  // this file, we'll change it.
  //
  // This is a lot better in the image links, since the defaultLink will just
  // download the image, but the one that we set it to will format the page
  // to have an image tag in the center (looking a lot like this text viewer)
  $Args[6] = $Args[1]['attachment_common']['href'];
}


function attachment_common_link_html(&$Args)
{
  $Args[1]['attachment_common']['href'] = '../src/download.php?startMessage=' . 
     $Args[2] . '&passed_id=' . $Args[3] . '&mailbox=' . $Args[4] . '&passed_ent_id=' .
     $Args[5] . '&override_type0=text&override_type1=html';
  
  if ($Args[8] && $Args[9])
     $Args[1]['attachment_common']['href'] .= '&where=' . 
     urlencode($Args[8]) . '&what=' . urlencode($Args[9]);

  $Args[1]['attachment_common']['text'] = _("view");
  
  $Args[6] = $Args[1]['attachment_common']['href'];
}


function attachment_common_options() {
   global $username, $data_dir;
   
?><tr><td align=right valign=top>View images inline:</td>
<td valign=top><input type=checkbox name=attachment_common_show_images<?PHP
   if (getPref($data_dir, $username, 'attachment_common_show_images'))
      echo ' CHECKED';
?>> Enable display of images with email, below attachment box</td></tr>
<?PHP
}


function attachment_common_options_save() {
   global $username, $data_dir, $attachment_common_show_images;
   
   if (isset($attachment_common_show_images))
      setPref($data_dir, $username, 'attachment_common_show_images', 1);
   else
      setPref($data_dir, $username, 'attachment_common_show_images', '');
}


function attachment_common_read_body_bottom()
{
   global $attachment_common_show_images, $username, $data_dir;
   
   if (! getPref($data_dir, $username, 'attachment_common_show_images'))
      return;
   if (! isset($attachment_common_show_images) ||
       ! is_array($attachment_common_show_images))
      return;

   foreach ($attachment_common_show_images as $img) {
?>
<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 ALIGN=CENTER>
  <TR>
    <TD>
      <img src="../src/download.php?passed_id=<?PHP echo
urlencode($img['passed_id']) ?>&mailbox=<?PHP echo urlencode($img['mailbox'])
?>&passed_ent_id=<?PHP echo urlencode($img['ent_id']) 
?>&absolute_dl=true">
    </TD>
  </TR>
</TABLE>
<?PHP
   }
}


function attachment_common_link_image(&$Args)
{
  global $attachment_common_show_images;
  
  $info['passed_id'] = $Args[3];
  $info['mailbox'] = $Args[4];
  $info['ent_id'] = $Args[5];
  
  $attachment_common_show_images[] = $info;
  
  $Args[1]['attachment_common']['href'] = '../plugins/attachment_common/image.php?startMessage=' .
     $Args[2] . '&passed_id=' . $Args[3] . '&mailbox=' . $Args[4] . '&passed_ent_id=' .
     $Args[5];
  
  if ($Args[8] && $Args[9])
     $Args[1]['attachment_common']['href'] .= '&where=' . 
     urlencode($Args[8]) . '&what=' . urlencode($Args[9]);

  $Args[1]['attachment_common']['text'] = _("view");
  
  $Args[6] = $Args[1]['attachment_common']['href'];
}


function attachment_common_link_vcard(&$Args)
{
  $Args[1]['attachment_common']['href'] = '../plugins/attachment_common/vcard.php?startMessage=' .
     $Args[2] . '&passed_id=' . $Args[3] . '&mailbox=' . $Args[4] . '&passed_ent_id=' .
     $Args[5];

  if (isset($where) && isset($what))
     $Args[1]['attachment_common']['href'] .= '&where=' . 
     urlencode($Args[8]) . '&what=' . urlencode($Args[9]);

  $Args[1]['attachment_common']['text'] = _("Business Card");

  $Args[6] = $Args[1]['attachment_common']['href'];
}


function attachment_common_octet_stream(&$Args)
{
   global $FileExtensionToMimeType;
   
   do_hook('attachment_common-load_mime_types');
   
   ereg('\\.([^\\.]+)$', $Args[7], $Regs);
  
   $Ext = strtolower($Regs[1]);
   
   if ($Ext == '' || ! isset($FileExtensionToMimeType[$Ext]))
       return;       
   
   $Ret = do_hook('attachment ' . $FileExtensionToMimeType[$Ext], 
       $Args[1], $Args[2], $Args[3], $Args[4], $Args[5], $Args[6], 
       $Args[7], $Args[8], $Args[9]);
       
   foreach ($Ret as $a => $b) {
       $Args[$a] = $b;
   }
}

?>
