<?

function squirrelmail_plugin_init_autocomplete()
{
   global $squirrelmail_plugin_hooks;

   $squirrelmail_plugin_hooks['compose_bottom']['autocomplete']    
      = 'plugin_autocomplete_compose_bottom';
   $squirrelmail_plugin_hooks['options_display_inside']['autocomplete'] 
      = 'plugin_autocomplete_display_inside';
   $squirrelmail_plugin_hooks['options_display_save']['autocomplete'] 
      = 'plugin_autocomplete_display_save';
   $squirrelmail_plugin_hooks['loading_prefs']['autocomplete'] 
      = 'plugin_autocomplete_loading_prefs';
}   
   

function plugin_autocomplete_compose_bottom()
{
   global $autocomplete_enabled;
   global $autocomplete_insensitive;
   
   if ($autocomplete_enabled == "None")
      return;
      
   include ('../functions/addressbook.php');

   // Open addressbook without error messages and without LDAP
   $abook = addressbook_init(false, true);
   if ($abook->localbackend == 0)
      // No personal address book is defined
      return;
      
   $alist = $abook->list_addr();
   if (! is_array($alist))
      // Some sort of error
      return;
   
   $addrs = array();
   
   // Nicknames
   if ($autocomplete_enabled == 'Both' || $autocomplete_enabled == 'Alias') {
      foreach ($alist as $val) {
         $addrs[] = $val['nickname'];
      }
   }
   
   // Emails
   if ($autocomplete_enabled == 'Both' || $autocomplete_enabled == 'Email') {
      foreach ($alist as $val) {
         $addrs[] = $val['email'];
      }
   }
   
   // If we don't have a list, do not spit out the javascript
   // (saves download time)
   if (count($addrs) == 0)
      return;
      
   // Now we have a big list of things we want to autocomplete.
   sort($addrs);
   
   // Spit out javascript
   echo '<script language="Javascript">
var autocompleteArray = new Array("' . join("\",\n\"", $addrs) . '");

function autocomplete_find (str) {
   var l = autocompleteArray.length;
   var strl = str.length;
   var sValue = "";
   for (var i = 0; i < l; i ++) {
      sValue = autocompleteArray[i];
';
if ($autocomplete_insensitive)
   echo '      // Case insensitive compare
      if (sValue.substring(0, strl).toLowerCase() == str.toLowerCase()) {
';
else
   echo '      // Case sensitive compare
      if (sValue.substring(0, strl) == str) {
';
   echo '         return sValue;
      }
   }
   return "";
}

function autocomplete_work (src) {
   if (! src.createTextRange)
      return;
      
   // Ignore cursor keys, shift, alt, etc
   // look only for A-Z, 0-9, etc.
   if (event.keyCode < 48 || (event.keyCode > 57 && event.keyCode < 65) ||
       (event.keyCode > 90 && event.keyCode < 96) || event.keyCode == 108 || 
       (event.keyCode > 111 && event.keyCode < 186) ||
       (event.keyCode > 192 && event.keyCode < 219) || event.keyCode > 222)
      return;
      
   // If there is no change in the field, ignore
   if (src.value == src._value)
      return;

   // kinda support multiple addresses
   var r1 = src.createTextRange();
   var Str = r1.text;
   var StartPos = 0;
   var newSP = Str.indexOf(" ");
   while (newSP >= StartPos) {
      StartPos = newSP + 1;
      newSP = Str.indexOf(" ", StartPos);
   }
      
   // do not search for nothing
   if (Str.length - StartPos <= 0)
      return;

   // If we can find something in our lookup list
   newValue = autocomplete_find(Str.substr(StartPos, Str.length));
   if (newValue == "" || newValue == Str.substr(StartPos, Str.length))
      return;
      
   newValue = Str.substr(0, StartPos) + newValue;
   var pos = Str.length;
   src.value = newValue;
   var rNew = src.createTextRange();
   rNew.moveStart("character", pos);
   rNew.select();
   src._value = src.value;
}

</script>
<script FOR=send_to EVENT=onkeyup>
autocomplete_work(this)
</script>
<script FOR=send_to_cc EVENT=onkeyup>
autocomplete_work(this)
</script>
<script FOR=send_to_bcc EVENT=onkeyup>
autocomplete_work(this)
</script>
';
}

function plugin_autocomplete_display_inside()
{
   global $username,$data_dir, $version;
   global $autocomplete_enabled, $autocomplete_insensitive;

   echo "<tr><td align=right valign=top>\n";
   echo "Autocomplete:</td>\n";
   echo "<td><select name=autocomplete_enabled_i>";
   echo "<option value=None";
   if ($autocomplete_enabled == "None")
      echo " SELECTED";
   echo ">Do not use autocomplete</option>\n";
   if (substr($version, 0, 4) != '1.0.' || substr($version, 0, 2) == '0.') {
      echo "<option value=Alias";
      if ($autocomplete_enabled == "Alias")
         echo " SELECTED";
      echo ">Address book nicknames</option>\n";
   }
   echo "<option value=Email";
   if ($autocomplete_enabled == "Email")
      echo " SELECTED";
   echo ">Address book email addresses</option>\n";
   if (substr($version, 0, 4) != '1.0.' || substr($version, 0, 2) == '0.') {
      echo "<option value=Both";
      if ($autocomplete_enabled == "Both")
         echo " SELECTED";
      echo ">Both nicknames and email addresses</option>\n";
   }
   echo "</select> (IE only)<br>\n";
   echo "<input type=checkbox name=autocomplete_insensitive_i";
   if ($autocomplete_insensitive)
      echo " CHECKED";
   echo "> Use case-insensitive searches";
   echo "</td></tr>\n";
}

function plugin_autocomplete_display_save()
{
   global $username,$data_dir;
   global $autocomplete_enabled_i, $autocomplete_insensitive_i;

   if (isset($autocomplete_enabled_i)) {
      setPref($data_dir, $username, 'autocomplete_enabled',
         $autocomplete_enabled_i);
   } else {
      setPref($data_dir, $username, 'autocomplete_enabled', '');
   }
   if (isset($autocomplete_insensitive_i)) {
      setPref($data_dir, $username, 'autocomplete_insensitive', 1);
   } else {
      setPref($data_dir, $username, 'autocomplete_insensitive', 0);
   }
}


function plugin_autocomplete_loading_prefs()
{
   global $username,$data_dir, $version;
   global $autocomplete_enabled, $autocomplete_insensitive;
   
   $autocomplete_enabled = getPref($data_dir, $username, 
      'autocomplete_enabled');

   $autocomplete_insensitive = getPref($data_dir, $username, 
      'autocomplete_insensitive');


   if ($autocomplete_enabled == '')
      $autocomplete_enabled = 'Alias';
      
   if ((substr($version, 0, 4) == "1.0." || substr($version, 0, 2) == '0.') &&
       ($autocomplete_enabled == 'Alias' || $autocomplete_enabled == 'Both'))
      $autocomplete_enabled = 'Email';
      
   
   if ($autocomplete_insensitive == '')
      $autocomplete_insensitive = 0;
}


