<?php


/* Initialize the plugin */
function squirrelmail_plugin_init_spamcop() {
   global $squirrelmail_plugin_hooks, $data_dir, $username, $spamcop_id;

   $squirrelmail_plugin_hooks['options_link_and_description']['spamcop'] =
      'spamcop_options';
   $squirrelmail_plugin_hooks['loading_prefs']['spamcop'] =
      'spamcop_load';
   $squirrelmail_plugin_hooks['read_body_header_right']['spamcop'] =
      'spamcop_show_link';
   $squirrelmail_plugin_hooks['options_identities_process']['spamcop'] =
      'spamcop_ident_process';
   $squirrelmail_plugin_hooks['options_identities_top']['spamcop'] =
      'spamcop_ident_top';
   $squirrelmail_plugin_hooks['options_identities_table']['spamcop'] = 
      'spamcop_ident_table';
   $squirrelmail_plugin_hooks['options_identities_buttons']['spamcop'] =
      'spamcop_ident_buttons';
   $squirrelmail_plugin_hooks['options_identities_renumber']['spamcop'] =
      'spamcop_ident_renumber';
   $squirrelmail_plugin_hooks['options_identities_default']['spamcop'] =
      'spamcop_ident_default';
      
   if (isset($spamcop_id)) {
      $squirrelmail_plugin_hooks['compose_send']['spamcop'] =
         'spamcop_while_sending';
   }
}


// Load the settings
// Validate some of it (make '' into 'default', etc.)
function spamcop_load() {
   global $username, $data_dir, $spamcop_enabled, $spamcop_delete,
      $spamcop_identity;

   $spamcop_enabled = getPref($data_dir, $username, 'spamcop_enabled');
   $spamcop_delete = getPref($data_dir, $username, 'spamcop_delete');
   $spamcop_identity = getPref($data_dir, $username, 'spamcop_identity');
   $idents = getPref($data_dir, $username, 'identities');
   $dirty = 0;
   if ($spamcop_identity == '') {
      $spamcop_identity = 'default';
      $dirty = 1;
   }
   if ($spamcop_identity != 'default' && $idents <= $spamcop_identity) {
      $spamcop_identity = 'default';
      $dirty = 1;
   }
   if ($dirty)
      setPref($data_dir, $username, 'spamcop_identity', $spamcop_identity);
}


// Show the link on the read-a-message screen
function spamcop_show_link() {
   global $passed_id, $mailbox, $ent_num, $spamcop_enabled;

   // This was stolen from printer_friendly
   // Do I really need/want it?
   if (!trim($mailbox))
      $mailbox = 'INBOX';

   if (! $spamcop_enabled)
      return;

   echo '<br><a href="../plugins/spamcop/spamcop.php?mailbox=' . 
      urlencode($mailbox) . 
      '&passed_id=' . urlencode($passed_id) . '">Report as Spam</a>' . "\n";
}


// Show the link to our own custom options page
function spamcop_options()
{
      global $color
      ?>
      <table width=50% cellpadding=3 cellspacing=0 border=0 align=center>
         <tr>
            <td bgcolor="<? echo $color[9] ?>">
               <a href="../plugins/spamcop/options.php">SpamCop - Spam Reporting</a>
            </td>
         </tr>
         <tr>
            <td bgcolor="<? echo $color[0] ?>">
	       Helps fight the battle against unsolicited email.  SpamCop
	       reads the spam email and determines the correct addresses to
	       send complaints to.  Quite fast, really smart, and easy to use.
            </td>
         </tr>
      </table>
      <?php
}


// When we send the email, we optionally trash it then too
function spamcop_while_sending()
{
   global $mailbox, $spamcop_delete, $spamcop_id, $auto_expunge, $username,
      $key, $imapServerAddress, $imapPort;

   if ($spamcop_delete) {
      $imapConnection = sqimap_login($username, $key, $imapServerAddress, 
         $imapPort, 0);
      sqimap_mailbox_select($imapConnection, $mailbox);
      sqimap_messages_delete($imapConnection, $spamcop_id, $spamcop_id, 
         $mailbox);
      if ($auto_expunge)
         sqimap_mailbox_expunge($imapConnection, $mailbox, true);
   }
}


// Our own custom processing routine.  This takes care of the Use For SpamCop
// button (that changes the identity that we use for SpamCop reports)
function spamcop_ident_process(&$Args) {
   global $spamcop_identity, $data_dir, $username;
   
   // [1] = Run SaveUpdateFunction (alterable)
   //       We don't use this, but if you were to $Args[1] = true;
   //       Then the SaveUpdateFunction would be ran to save changes.

   // Save the old identity
   $ident_old = $spamcop_identity;
   
   $i = 0;
   // I hate loops like this, but it is easier for this instance
   while (1) {
      if ($i == 0)
         $post = '';
      else
         $post = $i;
      
      // If there was a form for this identity (it exists)
      $name = 'form_for_' . $post;
      global $$name;
      if (! isset($$name)) {
         // Oh well, no form.  Quit here because we ran out of
	 // identities to check.
         if ($ident_old != $spamcop_identity) {
	    // Lookie!  We made a change!  Save it.
	    setPref($data_dir, $username, 'spamcop_identity', 
	       $spamcop_identity);
	 }
	 return;
      }
      
      // Identity exists.  Look to see if we pressed the button to select
      // this particular identity.
      $name = 'spamcop_select' . $post;
      global $$name;
      if (isset($$name)) {
         // We sure did
         if ($i == 0)
            $spamcop_identity = 'default';
	 else
	    $spamcop_identity = $i;
      }
      $i ++;
   }
}


// Little hook to display information, warnings, pictures, etc.
function spamcop_ident_top() {
   // warning message
?><p>You should create a temporary forwarding account to report spam with,
and add that account as an identity.  It is also a good idea to filter
messages to that account into a separate mailbox.  That way, if the forwarding 
account starts to get spam mail, you can just delete the account.</p><?PHP
}


// Adds rows to the table.  We wanna just add a message if this identity is
// the one selected for the SpamCop reports.
function spamcop_ident_table($Args) {
   global $spamcop_identity;

   // [1] = bgcolor    echo "<tr" . $Args[1] . ">";
   // [2] = is empty section
   // [3] = post

   // If this isn't it, we quit.
   // [3] is a number or 'default'
   if (! ($spamcop_identity == 'default' && $Args[3] == '') &&
       $spamcop_identity != $Args[3])
      return;
      
   // Output a full row (<tr><td></td><td></td></tr>)
   // Make sure that there is something in each cell!  Even if it is just
   // a &nbsp; character.
   echo '<tr' . $Args[1] . '><td>&nbsp;</td><td>Spamcop uses this ' .
      'identity to send spam reports</td></tr>' . "\n";
}


// Adds a button to the button table cell
// Only add if this isn't the current SpamCop identity
// Only add if this isn't the empty form
function spamcop_ident_buttons($Args) {
   global $spamcop_identity;
   
   // [1] = IsEmpty
   // [2] = post
   
   // If this is the empty box
   if ($Args[1])
      return;
      
   // If this is the current one
   if ($Args[2] == $spamcop_identity)
      return;
   
   // If this is the current one and it is the default identity
   if ($Args[2] == '' && $spamcop_identity == 'default')
      return;
      
?><input type=submit name="spamcop_select<?PHP echo $Args[2] 
?>" value="Use For SpamCop">
<?PHP
}


// This is called a lot -- do not output anything from here since it can
// be called right before a redirect header is output.
// If there is anything that changes (and lots of stuff that doesn't), then
// this hook is called.
function spamcop_ident_renumber($Args) {
   global $spamcop_identity, $username, $data_dir;

   // [1] = from
   // [2] = to
   
   // If they are the same, do not do anything.  No work is needed.
   if ($Args[1] == $Args[2])
      return;
      
   // If one of the two is the same as us, continue processing
   // NOTE:  'default' is used for the default entry, not ''
   if ($spamcop_identity != $Args[1] && $spamcop_identity != $Args[2])
      return;
   
   if ($spamcop_identity == $Args[1])
      $spamcop_identity = $Args[2];
   else
      $spamcop_identity = $Args[1];
   setPref($data_dir, $username, 'spamcop_identity', $spamcop_identity);
}

?>
